from __future__ import absolute_import

import warnings
import datetime as dt

import jsonpickle.pickler
import jsonpickle.unpickler
import bson.binary
from jaraco.util.string import is_binary

# load the specialized handlers
__import__('jaraco.modb.datetime')
__import__('jaraco.modb.odict')

# override the default pickler/unpickler to handle binary strings
class Pickler(jsonpickle.pickler.Pickler):
	def flatten(self, obj):
		if isinstance(obj, dt.datetime) and not obj.utcoffset():
			# naive datetimes or UTC datetimes can be stored directly
			return obj
		flattened = super(Pickler, self).flatten(obj)
		if is_binary(flattened):
			return bson.binary.Binary(flattened)
		return flattened

class Unpickler(jsonpickle.unpickler.Unpickler):
	def restore(self, obj):
		restored = super(Unpickler, self).restore(obj)
		if isinstance(restored, bson.binary.Binary):
			return bytes(restored)
		return restored

def init():
	warnings.warn("It is no longer necessary to call jaraco.modb.init",
		DeprecationWarning)

def encode(value):
	return Pickler().flatten(value)

def decode(value):
	return Unpickler().restore(value)



import sys
import collections

from . import handlers

@handlers.register
class OrderedDictHandler(handlers.SimpleReduceHandler):
    if sys.version_info > (2,7):
        _handles = collections.OrderedDict,



from __future__ import absolute_import

import datetime

import bson.binary
import bson.tz_util
import py.test

import jaraco.modb

def test_to_bson():
	sample = dict(
		a = u'a string',
		b = 'another string',
		c = 'some binary bytes\x00\xff',
	)
	res = jaraco.modb.encode(sample)
	assert res['a'] == sample['a']
	assert res['b'] == sample['b']
	assert isinstance(res['c'], bson.binary.Binary)
	assert jaraco.modb.decode(res) == sample

class TestObject(object):
	def __init__(self, val):
		self.val = val

def test_object():
	ob = TestObject(1)
	serialized = jaraco.modb.encode(ob)
	ob_res = jaraco.modb.decode(serialized)
	assert isinstance(ob_res, TestObject)
	assert ob_res is not ob
	assert ob.val == ob_res.val

def test_nested_object():
	ob_nested = TestObject('child')
	ob_parent = TestObject(ob_nested)
	serialized = jaraco.modb.encode(ob_parent)
	restored = jaraco.modb.decode(serialized)
	assert isinstance(restored, TestObject)
	assert isinstance(restored.val, TestObject)
	assert restored.val.val == 'child'

class MyDict(dict):
	pass

def test_encode_dict_subclass():
	d = MyDict(a=3, b=4)
	encoded = jaraco.modb.encode(d)
	assert 'MyDict' in str(encoded)
	decoded = jaraco.modb.decode(encoded)
	assert isinstance(decoded, MyDict)

def test_init():
	"Init should do nothing, not cause errors"
	jaraco.modb.init()

def test_ordered_dict():
	try:
		import collections
		ob = collections.OrderedDict(a=1, c=3, b=4)
	except Exception:
		py.test.skip("OrderedDict not available")
	serialized = jaraco.modb.encode(ob)
	restored = jaraco.modb.decode(serialized)
	assert isinstance(restored, collections.OrderedDict)
	assert restored.keys() == list('acb')
	assert restored.values() == [1, 3, 4]

def test_datetime_naive():
	now = datetime.datetime.utcnow()
	serialized = jaraco.modb.encode(now)
	assert isinstance(serialized, datetime.datetime)
	restored = jaraco.modb.decode(serialized)
	assert restored == now

def test_datetime_utc():
	now = datetime.datetime.utcnow().replace(tzinfo=bson.tz_util.utc)
	serialized = jaraco.modb.encode(now)
	assert isinstance(serialized, datetime.datetime)
	restored = jaraco.modb.decode(serialized)
	assert restored == now

def test_datetime_local():
	now = datetime.datetime.now().replace(tzinfo=bson.tz_util.FixedOffset(-60*5, 'EST'))
	serialized = jaraco.modb.encode(now)
	assert not isinstance(serialized, datetime.datetime)
	restored = jaraco.modb.decode(serialized)
	assert restored == now



from __future__ import absolute_import

import datetime

import jsonpickle.handlers
import jsonpickle.util

from . import handlers

# hack to disable REPR for datetime objects in jsonpickle 0.4.0:
jsonpickle.util.NEEDS_REPR = ()


@handlers.register
class DatetimeHandler(jsonpickle.handlers.BaseHandler):
    """
    Datetime objects use __reduce__, and they generate binary strings encoding
    the payload. This handler encodes that payload to reconstruct the
    object.
    """
    _handles = datetime.datetime, datetime.date, datetime.time
    def flatten(self, obj, data):
        pickler = self._base
        if not pickler.unpicklable:
            return unicode(obj)
        cls, args = obj.__reduce__()
        args = [args[0].encode('base64')] + map(pickler.flatten, args[1:])
        data['__reduce__'] = (pickler.flatten(cls), args)
        return data

    def restore(self, obj):
        cls, args = obj['__reduce__']
        value = args[0].decode('base64')
        unpickler = self._base
        cls = unpickler.restore(cls)
        params = map(unpickler.restore, args[1:])
        params = (value,) + tuple(params)
        return cls.__new__(cls, *params)

@handlers.register
class TimeDeltaHandler(handlers.SimpleReduceHandler):
    _handles = datetime.timedelta,



import jsonpickle.handlers

def register(class_):
    for handled_type in getattr(class_, '_handles', []):
        jsonpickle.handlers.registry.register(handled_type, class_)
    return class_

class SimpleReduceHandler(jsonpickle.handlers.BaseHandler):
    """
    Follow the __reduce__ protocol to pickle an object. As long as the factory
    and its arguments are pickleable, this should pickle any object that
    implements the reduce protocol.
    """
    def flatten(self, obj, data):
        pickler = self._base
        if not pickler.unpicklable:
            return unicode(obj)
        data['__reduce__'] = map(pickler.flatten, obj.__reduce__())
        return data

    def restore(self, obj):
        unpickler = self._base
        cls, args = map(unpickler.restore, obj['__reduce__'])
        return cls(*args)



#!/usr/bin/env python
# Generated by jaraco.develop (https://bitbucket.org/jaraco/jaraco.develop)
import os

from setuptools import find_packages

with open(os.path.join(os.path.dirname(__file__), 'README')) as readme:
	long_description = readme.read()

setup_params = dict(
	name='jaraco.modb',
	description="MongoDB Object DataBase (MODB) for Python objects",
	long_description=long_description,
	use_hg_version=dict(increment='0.1'),
	packages=find_packages(),
	namespace_packages=['jaraco'],
	zip_safe=False,
	classifiers = [
		"Development Status :: 5 - Production/Stable",
		"Intended Audience :: Developers",
		"Programming Language :: Python :: 2.6",
		"Programming Language :: Python :: 2.7",
		"Programming Language :: Python :: 3",
	],
	install_requires=[
		'jsonpickle==0.4.0',
		'pymongo',
		'jaraco.util',
	],
	setup_requires=[
		'hgtools<8',
		'pytest-runner',
	],
)
if __name__ == '__main__':
	from setuptools import setup
	setup(**setup_params)
